from sys import argv
from moteur_chainage_arriere.connaissance import BaseConnaissances
from moteur_chainage_arriere.chainage_arriere import ChainageArriere
from moteur_avec_variables.regle_avec_variables import RegleAvecVariables
from moteur_avec_variables.unificateur import Unificateur

femmes = ['françoise', 'julie', 'alphonsine', 'véronique', 
          'charlotte', 'brigitte', 'constance']
hommes = ['jean', 'pierre', 'marc', 'philippe', 'gustave', 
          'octave', 'antoine', 'hyacynthe', 'rodolphe']
tous = femmes + hommes

faits = []
faits.extend([('femme', f) for f in femmes])
faits.extend([('homme', f) for f in hommes])
faits.extend([('différent', x, y) for x in tous for y in tous if x != y])

faits.extend([
    ('parent', 'jean', 'pierre'),
    ('parent', 'françoise', 'pierre'),
    ('parent', 'philippe', 'alphonsine'),
    ('parent', 'charlotte', 'alphonsine'),
    ('parent', 'pierre', 'julie'),
    ('parent', 'alphonsine', 'julie'),
    ('parent', 'antoine', 'véronique'),
    ('parent', 'constance', 'véronique'),
    ('parent', 'octave', 'marc'),
    ('parent', 'brigitte', 'marc'),
    ('parent', 'marc', 'gustave'),
    ('parent', 'véronique', 'gustave'),
    ('parent', 'gustave', 'hyacynthe'),
    ('parent', 'julie', 'hyacynthe'),
    ('parent', 'philippe', 'rodolphe'),
    ('parent', 'brigitte', 'rodolphe'),
])

regles = [
    [[('parent', '?x', '?y'), ('femme', '?x')], ('mère', '?x', '?y')],
    [[('parent', '?x', '?y'), ('homme', '?x')], ('père', '?x', '?y')],
    [[('parent', '?x', '?y')], ('enfant', '?y', '?x')],
    [[('parent', '?x', '?y'), ('homme', '?y')], ('fils', '?y', '?x')],
    [[('parent', '?x', '?y'), ('femme', '?y')], ('fille', '?y', '?x')],
    [[('parent', '?x', '?z'), ('parent', '?z', '?y')],
     ('grand-parent', '?x', '?y')],
    [[('parent', '?x', '?z'), ('parent', '?z', '?y'), ('parent', '?y', '?u')],
     ('arrière-grand-parent', '?x', '?u')],
    [[('grand-parent', '?x', '?y'), ('femme', '?x')], ('grand-mère', '?x', '?y')],
    [[('grand-parent', '?x', '?y'), ('homme', '?x')], ('grand-père', '?x', '?y')],
    [[('arrière-grand-parent', '?x', '?y'), ('femme', '?x')],
     ('arrière-grand-mère', '?x', '?y')],
    [[('arrière-grand-parent', '?x', '?y'), ('homme', '?x')],
     ('arrière-grand-père', '?x', '?y')],
    [[('parent', '?x', '?z'), ('parent', '?x', '?y'), ('homme', '?y'),
      ('différent', '?z', '?y')], ('frère', '?y', '?z')],
    [[('parent', '?x', '?z'), ('parent', '?x', '?y'), ('femme', '?y'),
      ('différent', '?z', '?y')], ('soeur', '?y', '?z')],
    ]

questions = [
   ('arrière-grand-mère', '?qui', '?qui-d-autre'),
   ('arrière-grand-parent', '?qui', '?qui-d-autre'),
   ('soeur', '?qui', '?qui-d-autre'),
   ('frère', '?qui', '?qui-d-autre'),
   ('fils', '?qui', '?qui-d-autre'),
   ('fils', 'gustave', 'marc'),
   ('fils', '?qui', 'marc'),
   ('fils', 'gustave', 'hyacynthe'),
   ('fils', 'gustave', 'ferdinand'),
]

bc = BaseConnaissances(lambda descr: RegleAvecVariables(descr[0], descr[1]))
bc.ajoute_faits(faits)
bc.ajoute_regles(regles)

moteur = ChainageArriere(bc, Unificateur())

for question in questions:

    print()
    print('Question: ' + str(question))

    moteur.reinitialise()
    moteur.chaine(question)

    moteur.affiche_solutions()

    if len(argv) > 1 and argv[1].lower() == 'trace':
        moteur.affiche_trace()