from moteur_sans_variables.chainage import Chainage
from .noeud import Noeud
from .noeuds_testes import NoeudsTestes

class ChainageArriere(Chainage):
    """ Un moteur d'inférence à chaînage arrière. """

    def __init__(self, connaissances, unificateur):
        self.connaissances = connaissances
        self.unificateur = unificateur

    def successeurs(self, noeud):
        """ Trouve les successeurs d'un noeud. """
        nouveaux_noeuds = []
        
        if noeud.est_terminal():
            return nouveaux_noeuds
            
        # On se limite aux faits et aux règles intéressants à examiner.
        # En effet, si le sous-but examiné commence par 'grand-pere',
        # alors il est inutile de considérer des noeuds commençant
        # par 'oncle' par exemple.
        regles_interessantes = self.connaissances.choisir_regles_interessantes(noeud.sous_but_courant, self.unificateur)
        faits_interessants = self.connaissances.choisir_faits_interessants(noeud.sous_but_courant)

        # On parcourt les règles intéressantes.
        for regle in regles_interessantes:
            # Tentative d'unification entre le sous-but sélectionné
            # et la consequence de la règle.
            env = self.unificateur.unifie(noeud.sous_but_courant, 
                                          regle.conclusion)
            if env != self.unificateur.echec:
                nouveau_noeud = noeud.successeur(env, 
                                                 regle.conditions, 
                                                 self.unificateur)
                nouveaux_noeuds.append(nouveau_noeud)

        # On parcourt les faits intéressants.
        for fait in faits_interessants:
            # Tentative d'unification entre le sous-but sélectionné et le fait.
            env = self.unificateur.unifie(noeud.sous_but_courant, fait)
            if env != self.unificateur.echec:
                nouveau_noeud = noeud.successeur(env, [], self.unificateur)
                nouveaux_noeuds.append(nouveau_noeud)
        # Retourne les nouveaux noeuds ainsi trouvés.
        return nouveaux_noeuds

    def backchain(self, noeud_depart):
        """ Exécute l'algorithme de chaînage arrière afin de trouver les\
            faits qui peuvent s'unifier avec le pattern.
        """
        # Liste des noeuds à tester.
        queue = [noeud_depart]
        # Liste des noeuds déjà testés.
        noeuds_testes = NoeudsTestes()
        # Liste des solutions.
        self.solutions = set()

        # Tant qu'il y a des noeuds à tester dans la liste,
        while len(queue) > 0:
            # on sélectionne le noeud suivant.
            noeud = queue.pop(0)
            self.trace.append(noeud)
            # Si le noeud n'appartient pas encore à la liste des noeuds 
            # déjà testés, on l'y ajoute pour éviter les cycles.
            if noeud not in noeuds_testes:
                noeuds_testes.ajoute(noeud)
                # Si le noeud est une solution, on l'ajoute à celles qu'on a 
                # déjà trouvées.
                if noeud.est_solution():
                    self.solutions.add(noeud.but)
                else:
                    # On obtient des noeuds supplémentaires par chaînage arrière.
                    successeurs = self.successeurs(noeud)
                    successeurs.extend(queue)
                    queue = successeurs

        # Retourne la liste des solutions au problème.
        return self.solutions

    def chaine(self, pattern):
        """ Interroge le moteur de chaînage arrière (fonction d'interface). """

        # Retourne les solutions par chaînage arrière.
        noeud_depart = Noeud(pattern, pattern, [], 0)
        solutions = self.backchain(noeud_depart)

        return solutions