from moteur_avec_variables.proposition_avec_variables import *

class Noeud:
    """ Représentation d'un noeud. """

    def __init__(self, but, sous_but_courant, sous_buts_a_tester, profondeur):
        self.but = but
        self.sous_but_courant = sous_but_courant
        self.sous_buts_a_tester = sous_buts_a_tester
        self.profondeur = profondeur

    def est_terminal(self):
        """ Teste si le noeud courant est terminal.
         
            Un noeud est terminal lorsqu'il ne peut
            avoir de successeurs quels que soient les
            faits et les règles.
        """
        return len(self.sous_but_courant) == 0 and len(self.sous_buts_a_tester) == 0

    def est_solution(self):
        """ Teste si le noeud courant est une solution. """
        variables = lister_variables(self.but)
        return self.est_terminal() and len(variables) == 0

    def successeur(self, env, nouveaux_sous_buts, unificateur):
        """ Retourne un noeud successeur du noeud courant. """
        # Les sous-buts à explorer sont composés par les sous-buts
        # encore ouverts et les nouveaux sous-buts (cas d'une règle).
        sous_buts = nouveaux_sous_buts[:]
        sous_buts.extend(self.sous_buts_a_tester)

        if len(sous_buts) > 0:
            premier_sous_but = sous_buts[0] 
        else:
            premier_sous_but = ()

        if len(sous_buts) > 1:
            reste_sous_buts = sous_buts[1:]
        else:
            reste_sous_buts = []

        successeur = Noeud(
                         unificateur.substitue(self.but, env),
                         unificateur.substitue(premier_sous_but, env),
                         [unificateur.substitue(sous_but, env) for sous_but in reste_sous_buts],
                         self.profondeur + 1
                        )

        return successeur

    def description_standardisee(noeud):
        """ Retourne une description du noeud sous forme de liste composée du\
            but en premier élément, puis des sous-buts triés selon l'ordre de\
            leurs propres descriptions.
        """

        sous_buts = [noeud.sous_but_courant] if len(noeud.sous_but_courant) > 0 else []
        sous_buts.extend(noeud.sous_buts_a_tester)
        sous_buts = sorted(sous_buts, key=lambda prop: prop)
        but_et_sous_buts = [noeud.but] + sous_buts

        return but_et_sous_buts

    def __repr__(self):
        return '<{},{},{},{}>'.format(self.but,
                                      self.sous_but_courant,
                                      self.sous_buts_a_tester,
                                      self.profondeur)